"use strict";
/*!
 * Copyright (c) 2019 Contributors to the Eclipse Foundation
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information regarding copyright ownership.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const https = require("https");
/**
 * NodeJs implementation of a Http Requester.
 */
class NodeRequester {
    constructor(agent) {
        this.proxyAgent = agent.proxyAgent;
    }
    static createResponseHandler(resolve, reject) {
        return response => {
            this.handleResponse(resolve, reject, response);
        };
    }
    static handleResponse(resolve, reject, response) {
        let data = '';
        response.on('data', chunk => {
            data += chunk;
        });
        response.on('end', () => {
            let body;
            if (data === '') {
                body = {};
            }
            else {
                body = JSON.parse(data);
            }
            const status = response.statusCode !== undefined ? response.statusCode : 0;
            if (status < 200 || status >= 300) {
                reject(body);
            }
            const headersObj = response.headers;
            const headers = Object.keys(headersObj).reduce((map, name) => {
                map.set(name, headersObj[name]);
                return map;
            }, new Map());
            resolve({ status, headers, body });
        });
    }
    doRequest(method, url, header, payload) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise(((resolve, reject) => {
                const req = https.request(this.buildRequest(method, url, header, payload), NodeRequester.createResponseHandler(resolve, reject));
                req.on('error', e => {
                    throw new Error(String(e));
                });
                if (payload !== undefined) {
                    req.write(payload);
                }
                req.end();
            }));
        });
    }
    /**
     * Builds an options object to perform a Http request with.
     *
     * @param method - The type of action to perform.
     * @param url - The Url to send the request to.
     * @param header - The headers of the request.
     * @param body - The payload to send with the request.
     * @return the builder.
     */
    buildRequest(method, url, header, body) {
        if (body !== undefined && body !== '') {
            header.set('Content-Length', Buffer.byteLength(body).toString());
        }
        const headers = {};
        header.forEach((v, k) => headers[k] = v);
        const parsedUrl = new URL(url);
        const request = {
            method,
            headers,
            host: parsedUrl.host,
            path: parsedUrl.pathname
        };
        if (this.proxyAgent !== undefined && this.proxyAgent.options.path !== undefined) {
            request['agent'] = this.proxyAgent;
        }
        return request;
    }
}
exports.NodeRequester = NodeRequester;
//# sourceMappingURL=node-http.js.map